/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@ricochet.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2004 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.ui.conf;

import org.apache.log4j.Logger;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.db.PostLinkDecoder;
import com.silverwrist.venice.except.*;
import com.silverwrist.venice.ui.*;
import com.silverwrist.venice.ui.helpers.*;
import com.silverwrist.venice.ui.servlet.BaseServlet;

public class TrackbackServlet extends BaseServlet
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Logger logger = Logger.getLogger(TrackbackServlet.class);

  private static final int INVALID = 0;
  private static final int COMMUNITY = 1;
  private static final int CONFERENCE = 2;
  private static final int TOPIC = 3;
  private static final int POST = 4;
  private static final int POST_RANGE = 5;
  private static final int POST_OE_RANGE = 6;

  /*--------------------------------------------------------------------------------
   * Internal operations
   *--------------------------------------------------------------------------------
   */

  private static final int classifyPostLink(RequestInput req, PostLinkDecoder pld)
  {
    String s = pld.getCommunity();
    if (s==null)
      return INVALID;  // we must have the community or we're not valid
    CommunityContext comm = null;
    try
    { // get the community context we're looking for
      comm = req.getUser().getCommunityContext(s);
      req.setRequestAttribute("current.community",comm);

    } // end try
    catch (DataException e)
    { // can't find the community - we're screwed
      logger.error("unable to decode community \"" + s + "\"",e);
      return INVALID;

    } // end catch

    s = pld.getConference();
    if (s==null)
      return COMMUNITY;  // community link only

    ConferenceContext conf = null;
    try
    { // get the conference context
      conf = comm.getConferenceContext(s);
      req.setRequestAttribute("current.conference",comm);

    } // end try
    catch (DataException e)
    { // can't find the conference - we're screwed
      logger.error("unable to decode conference \"" + s + "\"",e);
      return INVALID;

    } // end catch
    catch (AccessError ae)
    { // we can't get to the conference...
      logger.error("AccessError getting conference \"" + s + "\"",ae);
      return INVALID;

    } // end catch

    short tn = pld.getTopic();
    if (tn==-1)
      return CONFERENCE;  // only a conference locator

    TopicContext topic = null;
    try
    { // get the resulting topic number
      topic = conf.getTopic(tn);
      req.setRequestAttribute("current.topic",comm);

    } // end try
    catch (DataException e)
    { // we can't find the topic - we're screwed
      logger.error("unable to decode topic \"" + tn + "\"",e);
      return INVALID;

    } // end catch
    catch (AccessError ae)
    { // we can't get to the topic...
      logger.error("AccessError getting topic \"" + tn + "\"",ae);
      return INVALID;

    } // end catch

    int fp = pld.getFirstPost();
    if (fp==-1)
      return TOPIC;  // just referencing the topic
    int lp = pld.getLastPost();
    if (fp==lp)
      return POST;  // single post
    if (lp==-1)
      return POST_OE_RANGE;  // open-ended range of posts
    return POST_RANGE;  // normal range of posts

  } // end classifyPostLink

  /*--------------------------------------------------------------------------------
   * Implementations from class BaseServlet
   *--------------------------------------------------------------------------------
   */

  public Object process(RequestInput req)
  {
    String raw_link = req.getPathInfo().substring(1);
    PostLinkDecoder decoder;

    if (logger.isDebugEnabled())
      logger.debug("decoding post link: " + raw_link);

    try
    { // attempt to decode the path link information
      decoder = new PostLinkDecoder(raw_link);

    } // end try
    catch (ValidationException e)
    { // the post link decoder failed
      logger.error("Post link decode failed!",e);
      return new HTTPError(400,"Invalid post link \"" + raw_link + "\": " + e.getMessage());

    } // end catch

    // classify the post link and decode as much of it as we can
    int what = classifyPostLink(req,decoder);
    if (what==INVALID)
      return new HTTPError(400,"Invalid post link \"" + raw_link + "\"");
      
    // at this point, trackback may only be applied to single posts, not ranges or anything higher
    if (what!=POST)
      return new HTTPError(400,"Trackback only applies to single posts");

    TopicMessageContext msg = null;
    try
    { // get the message involved
      TopicContext topic = (TopicContext)(req.getRequestAttribute("current.topic"));
      msg = topic.getMessage(decoder.getFirstPost());

    } // end try
    catch (DataException e)
    { // the post was not found
      return new HTTPError(404,"Post not found: \"" + raw_link + "\"");

    } // end catch
    catch (AccessError e)
    { // could not access the post
      return new HTTPError(403,"Post not accessible: \"" + raw_link + "\"");

    } // end catch

    // TODO: POST = add trackback
    // GET = get list of trackbacks, may be with either ?__mode=rss (default) or ?__mode=view

    return null;

  } // end process

} // end class TrackbackServlet
